@@ -45,6 +45,12 @@
 
 static gboolean editor_mode = FALSE;
 
+enum {
+    BOOKMARKS_CHANGED,
+    LAST_SIGNAL
+};
+static gint signals[LAST_SIGNAL] = { 0 };
+
 static const GOptionEntry entries[] = {
     {"editor-mode", 0, 0, G_OPTION_ARG_NONE, &editor_mode, N_("Turn on editor mode"), NULL},
     { NULL }
@@ -68,11 +74,16 @@ static void          application_uri_resolved          (YelpUri               *u
 static gboolean      application_window_deleted        (YelpWindow            *window,
                                                         GdkEvent              *event,
                                                         YelpApplication       *app);
+GSettings *          application_get_doc_settings      (YelpApplication       *app,
+                                                        const gchar           *doc_uri);
 static gboolean      application_maybe_quit            (YelpApplication       *app);
 static void          application_adjust_font           (GtkAction             *action,
                                                         YelpApplication       *app);
 static void          application_set_font_sensitivity  (YelpApplication       *app);
 
+static gboolean      window_resized                    (YelpWindow            *window,
+                                                        YelpApplication       *app);
+
 G_DEFINE_TYPE (YelpApplication, yelp_application, G_TYPE_OBJECT);
 #define GET_PRIV(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_APPLICATION, YelpApplicationPrivate))
 
@@ -85,7 +96,9 @@ struct _YelpApplicationPrivate {
 
     GtkActionGroup *action_group;
 
+    gchar *gsettings_context; /* static, do not free */
     GSettings *gsettings;
+    GHashTable *docsettings;
 };
 
 static const GtkActionEntry action_entries[] = {
@@ -104,6 +117,10 @@ static const GtkActionEntry action_entries[] = {
 static void
 yelp_application_init (YelpApplication *app)
 {
+    YelpApplicationPrivate *priv = GET_PRIV (app);
+    priv->docsettings = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                               (GDestroyNotify) g_free,
+                                               (GDestroyNotify) g_object_unref);
 }
 
 static void
@@ -114,6 +131,14 @@ yelp_application_class_init (YelpApplicationClass *klass)
     object_class->dispose = yelp_application_dispose;
     object_class->finalize = yelp_application_finalize;
 
+    signals[BOOKMARKS_CHANGED] =
+        g_signal_new ("bookmarks-changed",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      0, NULL, NULL,
+                      g_cclosure_marshal_VOID__STRING,
+                      G_TYPE_NONE, 1, G_TYPE_STRING);
+
     dbus_g_object_type_install_info (YELP_TYPE_APPLICATION,
                                      &dbus_glib_yelp_object_info);
 
@@ -149,6 +174,7 @@ yelp_application_finalize (GObject *object)
     YelpApplicationPrivate *priv = GET_PRIV (object);
 
     g_hash_table_destroy (priv->windows_by_document);
+    g_hash_table_destroy (priv->docsettings);
 
     G_OBJECT_CLASS (yelp_application_parent_class)->finalize (object);
 }
@@ -170,8 +196,10 @@ application_setup (YelpApplication *app)
         gchar *keyfile = g_build_filename (g_get_user_config_dir (),
                                            "yelp", "yelp.cfg",
                                            NULL);
-        g_settings_backend_setup_keyfile ("yelp-", keyfile);
-        priv->gsettings = g_settings_new_with_context ("org.gnome.yelp", "yelp-");
+        g_settings_backend_setup_keyfile ("yelp", keyfile);
+        priv->gsettings_context = "yelp";
+        priv->gsettings = g_settings_new_with_context ("org.gnome.yelp",
+                                                       priv->gsettings_context);
         g_free (keyfile);
     }
     else {
@@ -411,13 +439,22 @@ application_uri_resolved (YelpUri             *uri,
         window = g_hash_table_lookup (priv->windows_by_document, doc_uri);
 
     if (window == NULL) {
+        gint width, height;
+        GSettings *settings = application_get_doc_settings (data->app, doc_uri);
+
+        g_settings_get (settings, "geometry", "(ii)", &width, &height);
         window = yelp_window_new (data->app);
+        gtk_window_set_default_size (GTK_WINDOW (window), width, height);
+        g_signal_connect (window, "resized", G_CALLBACK (window_resized), data->app);
         priv->windows = g_slist_prepend (priv->windows, window);
 
         if (!data->new) {
             g_hash_table_insert (priv->windows_by_document, doc_uri, window);
             g_object_set_data (G_OBJECT (window), "doc_uri", doc_uri);
         }
+        else {
+            g_free (doc_uri);
+        }
         g_signal_connect (window, "delete-event",
                           G_CALLBACK (application_window_deleted), data->app);
     }
@@ -439,6 +476,30 @@ application_uri_resolved (YelpUri             *uri,
     g_free (data);
 }
 
+GSettings *
+application_get_doc_settings (YelpApplication *app, const gchar *doc_uri)
+{
+    YelpApplicationPrivate *priv = GET_PRIV (app);
+    GSettings *settings = g_hash_table_lookup (priv->docsettings, doc_uri);
+    if (settings == NULL) {
+        gchar *tmp;
+        gchar *settings_path;
+        tmp = g_uri_escape_string (doc_uri, "", FALSE);
+        settings_path = g_strconcat ("/apps/yelp/documents/", tmp, "/", NULL);
+        g_free (tmp);
+        if (priv->gsettings_context)
+            settings = g_settings_new_with_context_and_path ("org.gnome.yelp.documents",
+                                                             priv->gsettings_context,
+                                                             settings_path);
+        else
+            settings = g_settings_new_with_path ("org.gnome.yelp.document",
+                                                 settings_path);
+        g_hash_table_insert (priv->docsettings, g_strdup (doc_uri), settings);
+        g_free (settings_path);
+    }
+    return settings;
+}
+
 static gboolean
 application_window_deleted (YelpWindow      *window,
                               GdkEvent        *event,
@@ -501,6 +562,55 @@ packages_installed (DBusGProxy     *proxy,
     }
 }
 
+void
+yelp_application_add_bookmark (YelpApplication   *app,
+                               const gchar       *doc_uri,
+                               const gchar       *page_id,
+                               const gchar       *icon,
+                               const gchar       *title)
+{
+    GSettings *settings;
+    YelpApplicationPrivate *priv = GET_PRIV (app);
+
+    settings = application_get_doc_settings (app, doc_uri);
+
+    if (settings) {
+        GVariantBuilder builder;
+        GVariantIter *iter;
+        gchar *this_id, *this_icon, *this_title;
+        gboolean broken = FALSE;
+        g_settings_get (settings, "bookmarks", "a(sss)", &iter);
+        g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(sss)"));
+        while (g_variant_iter_loop (iter, "(&s&s&s)", &this_id, &this_icon, &this_title)) {
+            if (g_str_equal (page_id, this_id)) {
+                /* Already have this bookmark */
+                broken = TRUE;
+                break;
+            }
+            g_variant_builder_add (&builder, "(sss)", this_id, this_icon, this_title);
+        }
+        g_variant_iter_free (iter);
+
+        if (!broken) {
+            GVariant *value;
+            g_variant_builder_add (&builder, "(sss)", page_id, icon, title);
+            value = g_variant_builder_end (&builder);
+            g_settings_set_value (settings, "bookmarks", value);
+            g_variant_unref (value);
+            g_signal_emit (app, signals[BOOKMARKS_CHANGED], 0, doc_uri, 0);
+        }
+    }
+}
+
+GVariant *
+yelp_application_get_bookmarks (YelpApplication *app,
+                                const gchar     *doc_uri)
+{
+    GSettings *settings = application_get_doc_settings (app, doc_uri);
+
+    return g_settings_get_value (settings, "bookmarks");
+}
+
 void
 yelp_application_install_package (YelpApplication  *app,
                                   const gchar      *pkg,
@@ -522,3 +632,29 @@ yelp_application_install_package (YelpApplication  *app,
                              G_TYPE_STRING, "",
                              G_TYPE_INVALID, G_TYPE_INVALID);
 }
+
+static gboolean
+window_resized (YelpWindow        *window,
+                YelpApplication   *app)
+{
+    YelpApplicationPrivate *priv = GET_PRIV (app);
+    gint width, height;
+    YelpUri *uri;
+    gchar *doc_uri;
+    GSettings *settings;
+
+    uri = yelp_window_get_uri (window);
+    doc_uri = yelp_uri_get_document_uri (uri);
+    settings = g_hash_table_lookup (priv->docsettings, doc_uri);
+
+    if (settings) {
+        gint width, height;
+        yelp_window_get_geometry (window, &width, &height);
+        g_settings_set (settings, "geometry", "(ii)", width, height);
+    }
+
+    g_free (doc_uri);
+    g_object_unref (uri);
+
+    return FALSE;
+}
